@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOUserAgent.java 1862365 2019-07-01 10:18:11Z ssteiner $ */
+/* $Id: FOUserAgent.java 1873298 2020-01-29 11:28:26Z ssteiner $ */
 
 package org.apache.fop.apps;
 
@@ -149,6 +149,7 @@ public class FOUserAgent {
         this.resourceResolver = resourceResolver;
         setTargetResolution(factory.getTargetResolution());
         setAccessibility(factory.isAccessibilityEnabled());
+        setKeepEmptyTags(factory.isKeepEmptyTags());
         imageSessionContext = new AbstractImageSessionContext(factory.getFallbackResolver()) {
 
             public ImageContext getParentContext() {
@@ -817,4 +818,17 @@ public class FOUserAgent {
     public HyphenationTreeCache getHyphenationTreeCache() {
         return factory.getHyphenationTreeCache();
     }
+
+    public void setKeepEmptyTags(boolean b) {
+        getRendererOptions().put(Accessibility.KEEP_EMPTY_TAGS, b);
+    }
+
+    public boolean isKeepEmptyTags() {
+        Boolean enabled = (Boolean)getRendererOptions().get(Accessibility.KEEP_EMPTY_TAGS);
+        if (enabled != null) {
+            return enabled;
+        } else {
+            return true;
+        }
+    }
 }
