@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FObj.java 1835810 2018-07-13 10:29:57Z ssteiner $ */
+/* $Id: FObj.java 1875645 2020-03-25 14:44:35Z ssteiner $ */
 
 package org.apache.fop.fo;
 
@@ -78,6 +78,8 @@ public abstract class FObj extends FONode implements Constants {
     private String layer;
     // End of property values
 
+    private boolean forceKeepTogether;
+
     /**
      * Create a new formatting object.
      *
@@ -775,6 +777,14 @@ public abstract class FObj extends FONode implements Constants {
         return (super.toString() + "[@id=" + this.id + "]");
     }
 
+    public boolean isForceKeepTogether() {
+        return forceKeepTogether;
+    }
+
+    public void setForceKeepTogether(boolean b) {
+        forceKeepTogether = b;
+    }
+
     /** Basic {@link FONode.FONodeIterator} implementation */
     public static class FObjIterator implements FONodeIterator {
 
