@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractIFPainter.java 1862426 2019-07-02 13:19:54Z ssteiner $ */
+/* $Id: AbstractIFPainter.java 1875656 2020-03-25 16:36:47Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -91,7 +91,7 @@ public abstract class AbstractIFPainter<T extends IFDocumentHandler> implements
      * Returns the intermediate format context object.
      * @return the context object
      */
-    protected IFContext getContext() {
+    public IFContext getContext() {
         return documentHandler.getContext();
     }
 
